@@ -8,8 +8,6 @@ FileDetailsDialog::FileDetailsDialog(QWidget *parent, QFileInfo info) :
 
     this->info = info;
 
-    user = getenv("USER");
-
     // Change color of descriptive labels
     QPalette labelPalette = ui->nameLabel->palette();
     labelPalette.setBrush(QPalette::Active, QPalette::WindowText, QBrush(QMaemo5Style::standardColor("SecondaryTextColor")));
@@ -35,8 +33,21 @@ FileDetailsDialog::FileDetailsDialog(QWidget *parent, QFileInfo info) :
     }
 
     // Size
-    qint64 size = info.size();
-    ui->sizeInfo->setText(FileSystemModel::sizeString(size) + QString(" (%1 B)").arg(size));
+    onSizeUpdated(info.size());
+    if (info.isDir()) {
+        sizeCounter = new SizeCounter(info);
+        connect(sizeCounter, SIGNAL(finished()), sizeCounter, SLOT(deleteLater()));
+        connect(sizeCounter, SIGNAL(finished()), this, SLOT(onSizeReady()));
+        connect(sizeCounter, SIGNAL(sizeUpdated(qint64,int,int)), this, SLOT(onSizeUpdated(qint64,int,int)));
+
+        // Setting the progress indicator does not work from the constructor,
+        // so it has to be delayed.
+        QTimer::singleShot(0, this, SLOT(onSizeStarted()));
+
+        sizeCounter->start();
+    } else {
+        sizeCounter = NULL;
+    }
 
     // Ownership
     ui->ownerInfo->setText(QString("%1:%2").arg(info.owner(), info.group()));
@@ -61,6 +72,7 @@ FileDetailsDialog::FileDetailsDialog(QWidget *parent, QFileInfo info) :
         ui->nameEdit->setEnabled(false);
 
     // Disable the permissions table if the file does not belong to us
+    QString user = getenv("USER");
     if (!(info.owner() == user || user == "root"))
         foreach (QCheckBox *box, ui->permissionsWidget->findChildren<QCheckBox*>())
             box->setEnabled(false);
@@ -83,6 +95,9 @@ FileDetailsDialog::FileDetailsDialog(QWidget *parent, QFileInfo info) :
 
 FileDetailsDialog::~FileDetailsDialog()
 {
+    if (sizeCounter)
+        sizeCounter->abort();
+
     delete ui;
 }
 
@@ -102,6 +117,31 @@ void FileDetailsDialog::onOrientationChanged()
     }
 }
 
+void FileDetailsDialog::onSizeStarted()
+{
+    this->setAttribute(Qt::WA_Maemo5ShowProgressIndicator, true);
+}
+
+void FileDetailsDialog::onSizeUpdated(qint64 size, int dirs, int files)
+{
+    QString text = FileSystemModel::sizeString(size)
+                 + QString(" (%L1 B)").arg(size);
+
+    if (dirs != -1)
+        text += QString("\n%1, %2")
+                .arg(tr("%Ln file(s)", "", files))
+                .arg(tr("%Ln subdir(s)", "", dirs));
+
+    ui->sizeInfo->setText(text);
+}
+
+void FileDetailsDialog::onSizeReady()
+{
+    sizeCounter = NULL;
+
+    this->setAttribute(Qt::WA_Maemo5ShowProgressIndicator, false);
+}
+
 void FileDetailsDialog::save()
 {
 
