@@ -14,6 +14,7 @@
 
 #include "ui_filedetailsdialog.h"
 #include "filesystemmodel.h"
+#include "sizecounter.h"
 
 namespace Ui {
     class FileDetailsDialog;
@@ -29,11 +30,14 @@ public:
 
 private:
     Ui::FileDetailsDialog *ui;
+    SizeCounter *sizeCounter;
     QFileInfo info;
-    QString user;
 
 private slots:
     void onOrientationChanged();
+    void onSizeStarted();
+    void onSizeUpdated(qint64 size, int dirs = -1, int files = 0);
+    void onSizeReady();
     void save();
 };
 
