@@ -213,7 +213,7 @@ FileOperation::Action FileOperation::checkOverwrite(QFileInfo source, QFileInfo
         || !target.isSymLink() && source.canonicalFilePath() == target.canonicalFilePath()) { // Link's source overwritten by the link
             switch (selfOverwriteAction) {
                 case Skip:
-                    // Ski[ this file
+                    // Skip this file
                     return Skip;
 
                 case Ask: default:
@@ -255,7 +255,7 @@ FileOperation::Action FileOperation::checkOverwrite(QFileInfo source, QFileInfo
 
         // Do not make unnecessary changes to the structure of directories
         if (source.isDir() && target.isDir())
-            return Skip;
+            return Merge;
 
         // Decide what to do if the file already exists
         switch (overwriteAction) {
@@ -396,7 +396,7 @@ void FileOperation::runCopyOrMove()
                 attentionHint = QString();
             }
 
-            // Take a possibility overwriting into account
+            // Take a possibility of overwriting into account
             switch (checkOverwrite(job->source, job->target)) {
                 case Allow: {
                     // We are allowed to use the target filename
@@ -531,6 +531,12 @@ void FileOperation::runCopyOrMove()
                     }
                     break;
                 }
+                case Merge: {
+                    // Merging directories, no special action required
+                    transferredSize += job->size;
+                    remainderDirs.prepend(job);
+                    iJob++;
+                }
                 default: {
                     break;
                 }
