@@ -56,7 +56,18 @@ void FileSystemModel::reload()
     if (currentDir.path() == ".") return;
 
     emit loadingStarted();
+
+    // Clear thumbnail jobs
+    foreach (ThumbnailJob *job, thumbnailJobs)
+        job->abort();
+    thumbnailJobs.clear();
+    currentThumbnailJob = 0;
+
+    // Clear buffered items
+    foreach (QStandardItem *item, itemBuffer)
+        delete item;
     itemBuffer.clear();
+
     model->clear();
 
     if (showDotDot && currentDir.path() != "/") {
@@ -71,6 +82,8 @@ void FileSystemModel::reload()
     if (currentDir.exists()) {
         FileSystemModelWorker *worker = new FileSystemModelWorker(this);
         connect(worker, SIGNAL(itemReady(QStandardItem*)), this, SLOT(onItemReady(QStandardItem*)));
+        connect(worker, SIGNAL(listReady()), this, SLOT(onListReady()));
+        connect(worker, SIGNAL(itemUpdated(int,int,QString)), this, SLOT(onItemUpdated(int,int,QString)));
         connect(worker, SIGNAL(finished()), this, SLOT(onWorkerFinished()));
 
         if (currentWorker)
@@ -83,27 +96,51 @@ void FileSystemModel::reload()
     }
 }
 
-
 void FileSystemModel::onItemReady(QStandardItem *item)
 {
     // Only results from the most recent job are interesting
     if (this->sender() == currentWorker) {
-        item->setIcon(showThumbnails ?
-                      FileSystemModel::thumbnail(item->data(PathRole).toString()):
-                      FileSystemModel::icon(item->data(PathRole).toString()));
+        item->setIcon(showThumbnails
+                      ? thumbnail(item->data(PathRole).toString(), item)
+                      : icon(item->data(PathRole).toString()));
         itemBuffer.append(item);
     } else {
         delete item;
     }
 }
 
+void FileSystemModel::onListReady()
+{
+    if (this->sender() == currentWorker) {
+        foreach (QStandardItem *item, itemBuffer)
+            model->appendRow(item);
+        itemBuffer.clear();
+
+        QTimer::singleShot(0, this, SLOT(processThumbnailJob()));
+
+        emit loadingFinished();
+    }
+}
+
+void FileSystemModel::processThumbnailJob()
+{
+    if (currentThumbnailJob < thumbnailJobs.count())
+        thumbnailJobs.at(currentThumbnailJob++)->start();
+
+    if (currentThumbnailJob < thumbnailJobs.count())
+        QTimer::singleShot(0, this, SLOT(processThumbnailJob()));
+}
+
+void FileSystemModel::onItemUpdated(int index, int role, QString detail)
+{
+    if (this->sender() == currentWorker)
+        model->item(index, 0)->setData(detail, role);
+}
+
 void FileSystemModel::onWorkerFinished()
 {
     if (this->sender() == currentWorker) {
         currentWorker = NULL;
-        while (!itemBuffer.isEmpty())
-            model->appendRow(itemBuffer.takeFirst());
-        emit loadingFinished();
     } else {
         workers.removeOne(static_cast<FileSystemModelWorker*>(this->sender()));
     }
@@ -178,7 +215,32 @@ void FileSystemModel::sortByType()
     reload();
 }
 
+// Return a thumbnail for the given file, if available, or queue creation
+QIcon FileSystemModel::thumbnail(QFileInfo info, QStandardItem *item)
+{
+    if (gchar* uri = g_filename_to_uri(info.absoluteFilePath().toUtf8(), NULL, NULL)) {
+        if (gchar* file = hildon_thumbnail_get_uri(uri, ThumbnailSize, ThumbnailSize, true)) {
+            // Simplified conversion of a URI (but apparently sufficient)
+            QString thumbFile = QString::fromUtf8(file).mid(7); // "file://"
+            g_free(file);
 
+            if (QFileInfo(thumbFile).exists()) {
+                // Thumbnail exists, return it immediately
+                g_free(uri);
+                return QIcon(thumbFile);
+            } else if (!info.isDir()) {
+                // Request thumbnail creation (but do not waste time on directories)
+                thumbnailJobs.append(new ThumbnailJob(uri, item));
+            }
+        }
+        g_free(uri);
+    }
+
+    // Fall back to an icon
+    return icon(info);
+}
+
+// Return an icon for the given file
 QIcon FileSystemModel::icon(QFileInfo info)
 {
         if (info.isDir()) {
@@ -197,36 +259,18 @@ QIcon FileSystemModel::icon(QFileInfo info)
         }
 }
 
-QIcon FileSystemModel::thumbnail(QFileInfo info)
-{
-    gchar* uri = g_filename_to_uri(info.absoluteFilePath().toUtf8(), NULL, NULL);
-    if (uri) {
-        gchar* file = hildon_thumbnail_get_uri(uri, 48, 48, true);
-        g_free(uri);
-
-        if (file) {
-            QString thumbFile = QString::fromUtf8(file).mid(7); // "file://"
-            g_free(file);
-
-            if (QFileInfo(thumbFile).exists())
-                return QIcon(thumbFile);
-        }
-    }
-    return icon(info);
-}
-
-QString FileSystemModel::sizeString(qint64 size)
+QString FileSystemModel::sizeString(double size)
 {
     const qint64 K = 1024;
     const qint64 M = 1024 * K;
     const qint64 G = 1024 * M;
 
-    if (size >= G*10)
-        return QString("%1 GiB").arg(size/G);
-    else if (size >= M*10)
-        return QString("%1 MiB").arg(size/M);
+    if (size >= G)
+        return QString("%1 GiB").arg(size/G, 0, 'f', 1);
+    else if (size >= M)
+        return QString("%1 MiB").arg(size/M, 0, 'f', 1);
     else if (size >= K)
-        return QString("%1 KiB").arg(size/K);
+        return QString("%1 KiB").arg(size/K, 0, 'f', 1);
     else
         return QString("%1 B").arg(size);
 }
