@@ -8,6 +8,7 @@
 #include <QFileSystemWatcher>
 #include <QDir>
 #include <QDateTime>
+#include <QTimer>
 
 #include <QThread>
 
@@ -16,6 +17,8 @@
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <gio/gio.h>
 
+#include "thumbnailjob.h"
+
 class FileSystemModelWorker;
 
 class FileSystemModel : public QSortFilterProxyModel
@@ -40,6 +43,8 @@ public:
         Permissions
     };
 
+    static const int ThumbnailSize = 48;
+
     FileSystemModel(QObject *parent);
 
     QString location();
@@ -55,9 +60,9 @@ public:
     void sortBySize();
     void sortByType();
 
+    QIcon thumbnail(QFileInfo info, QStandardItem *item);
     static QIcon icon(QFileInfo info);
-    static QIcon thumbnail(QFileInfo info);
-    static QString sizeString(qint64 size);
+    static QString sizeString(double size);
     static QString dateString(QDateTime date);
     static QString detailString(QFileInfo info, Detail detail);
 
@@ -82,11 +87,17 @@ private:
 
     FileSystemModelWorker *currentWorker;
     QList<FileSystemModelWorker*> workers;
+    QList<ThumbnailJob*> thumbnailJobs;
     QList<QStandardItem*> itemBuffer;
 
+    int currentThumbnailJob;
+
 private slots:
     void reload();
+    void processThumbnailJob();
     void onItemReady(QStandardItem *item);
+    void onListReady();
+    void onItemUpdated(int index, int role, QString value);
     void onWorkerFinished();
 };
 
@@ -100,11 +111,15 @@ public:
 
 signals:
     void itemReady(QStandardItem*);
+    void listReady();
+    void itemUpdated(int index, int role, QString value);
 
 private:
     QDir currentDir;
     FileSystemModel::Detail firstDetail;
     FileSystemModel::Detail secondDetail;
+
+    void updateSize(const QFileInfoList &entries, int role);
 };
 
 #endif // FILESYSTEMMODEL_H
