@@ -253,8 +253,10 @@ void LocationWidget::onButtonClicked()
         QAbstractButton *button = qobject_cast<QAbstractButton*>(buttonLayout->itemAt(i)->widget());
         if (button) {
             path.append(button->text());
-            if (button == this->sender())
+            if (button == this->sender()) {
+                button->setChecked(true);
                 break;
+            }
             path.append('/');
         }
     }
